import collections
import inspect
import sys

py2k = sys.version_info < (3, 0)
py3k = sys.version_info >= (3, 0)
py32 = sys.version_info >= (3, 2)
py27 = sys.version_info >= (2, 7)
jython = sys.platform.startswith("java")
win32 = sys.platform.startswith("win")

try:
    import threading
except ImportError:
    import dummy_threading as threading  # noqa

FullArgSpec = collections.namedtuple(
    "FullArgSpec",
    [
        "args",
        "varargs",
        "varkw",
        "defaults",
        "kwonlyargs",
        "kwonlydefaults",
        "annotations",
    ],
)

ArgSpec = collections.namedtuple(
    "ArgSpec", ["args", "varargs", "keywords", "defaults"]
)


def inspect_getfullargspec(func):
    """Fully vendored version of getfullargspec from Python 3.3."""

    if inspect.ismethod(func):
        func = func.__func__
    if not inspect.isfunction(func):
        raise TypeError("{!r} is not a Python function".format(func))

    co = func.__code__
    if not inspect.iscode(co):
        raise TypeError("{!r} is not a code object".format(co))

    nargs = co.co_argcount
    names = co.co_varnames
    nkwargs = co.co_kwonlyargcount if py3k else 0
    args = list(names[:nargs])
    kwonlyargs = list(names[nargs : nargs + nkwargs])

    nargs += nkwargs
    varargs = None
    if co.co_flags & inspect.CO_VARARGS:
        varargs = co.co_varnames[nargs]
        nargs = nargs + 1
    varkw = None
    if co.co_flags & inspect.CO_VARKEYWORDS:
        varkw = co.co_varnames[nargs]

    return FullArgSpec(
        args,
        varargs,
        varkw,
        func.__defaults__,
        kwonlyargs,
        func.__kwdefaults__ if py3k else None,
        func.__annotations__ if py3k else {},
    )


def inspect_getargspec(func):
    return ArgSpec(*inspect_getfullargspec(func)[0:4])


if py3k:  # pragma: no cover
    string_types = (str,)
    text_type = str
    string_type = str

    if py32:
        callable = callable  # noqa
    else:

        def callable(fn):  # noqa
            return hasattr(fn, "__call__")

    def u(s):
        return s

    def ue(s):
        return s

    import configparser
    import io
    import _thread as thread
else:
    # Using noqa bellow due to tox -e pep8 who use
    # python3.7 as the default interpreter
    string_types = (basestring,)  # noqa
    text_type = unicode  # noqa
    string_type = str

    def u(s):
        return unicode(s, "utf-8")  # noqa

    def ue(s):
        return unicode(s, "unicode_escape")  # noqa

    import ConfigParser as configparser  # noqa
    import StringIO as io  # noqa

    callable = callable  # noqa
    import thread  # noqa


if py3k or jython:
    import pickle
else:
    import cPickle as pickle  # noqa

if py3k:

    def read_config_file(config, fileobj):
        return config.read_file(fileobj)


else:

    def read_config_file(config, fileobj):
        return config.readfp(fileobj)


def timedelta_total_seconds(td):
    if py27:
        return td.total_seconds()
    else:
        return (
            td.microseconds + (td.seconds + td.days * 24 * 3600) * 1e6
        ) / 1e6



import os
import re
import sys

from setuptools import find_packages
from setuptools import setup
from setuptools.command.test import test as TestCommand


class UseTox(TestCommand):
    RED = 31
    RESET_SEQ = "\033[0m"
    BOLD_SEQ = "\033[1m"
    COLOR_SEQ = "\033[1;%dm"

    def run_tests(self):
        sys.stderr.write(
            "%s%spython setup.py test is deprecated by pypa.  Please invoke "
            "'tox' with no arguments for a basic test run.\n%s"
            % (self.COLOR_SEQ % self.RED, self.BOLD_SEQ, self.RESET_SEQ)
        )
        sys.exit(1)


v = open(os.path.join(os.path.dirname(__file__), "dogpile", "__init__.py"))
VERSION = (
    re.compile(r""".*__version__ = ["'](.*?)["']""", re.S)
    .match(v.read())
    .group(1)
)
v.close()

readme = os.path.join(os.path.dirname(__file__), "README.rst")

setup(
    name="dogpile.cache",
    version=VERSION,
    description="A caching front-end based on the Dogpile lock.",
    long_description=open(readme).read(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    keywords="caching",
    author="Mike Bayer",
    author_email="mike_mp@zzzcomputing.com",
    url="https://github.com/sqlalchemy/dogpile.cache",
    license="BSD",
    packages=find_packages(".", exclude=["tests*"]),
    entry_points="""
    [mako.cache]
    dogpile.cache = dogpile.cache.plugins.mako_cache:MakoPlugin
    """,
    zip_safe=False,
    install_requires=["decorator>=4.0.0"],
    cmdclass={"test": UseTox},
)
